#ifndef AG_ARRAY_H
#define AG_ARRAY_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include "agbarray.h"
#include <stddef.h>

template <class Object>
class AgArray
: public AgCopyControl< AgArrayBase<Object> >
{
  typedef AgArrayBase<Object>         ArrayBase;
  typedef AgCopyControl<ArrayBase>    Base;
  typedef AgArray<Object>             This;
public:
  typedef AgContents<Object, Base>   Contents;
  AgArray();
  AgArray(int n);
  AgArray(Object *x, int n);
  AgArray(const AgContainerContents<Object> &c);
  AgArray(const This &a);
  ~AgArray() {}

  This &operator = (const This &x) {Base::operator = (x); return *this;}
  This &operator = (const AgContainerContents<Object> &x) {
    *kernel() = x;
    return *this;
  }

  const Contents contents() const {return Contents(*this);}

  operator Object *() {return kernel()->operator Object *();}
  operator const Object *() const {return kernel()->operator const Object *();}

  Object &operator [] (int x) {return kernel()->operator[](x);}
  const Object &operator [] (int x) const {return kernel()->operator [](x);}

  int size() const {return kernel()->size();}
  This &resize(int n) {kernel()->resize(n); return *this;}
};

template <class Object>
AgArray<Object>::AgArray()
: AgCopyControl< AgArrayBase<Object> >(new AgArrayBase<Object>())
{
  // Nothing to do
}

template <class Object>
AgArray<Object>::AgArray(int n)
: Base(new ArrayBase(n))
{
  // Nothing to do
}

template <class Object>
AgArray<Object>::AgArray(Object *x, int n)
: Base(new ArrayBase(x, n))
{
  // Nothing to do
}

template <class Object>
AgArray<Object>::AgArray(const This &a)
: Base(a)
{
  // Nothing to do
}


template <class Object>
AgArray<Object>::AgArray(const AgContainerContents<Object> &c)
: Base(new ArrayBase(c))
{
  // Nothing to do
}

// General Container function definition

template <class Object, class Container>
AgArray<Object> &operator += (AgArray<Object> &a, const Container &c) {
  int n = c.size();
  int m = a.size();
  a.resize(n + m);
  Object *array = a;
  for (int i = 0; i < n; i++) array[i+m] = (Object &) c[i];
  return a;
}

// Hash functions for arrays

template <class Object>
inline int agABTHash(const AgArray<Object> &c, int hash) {
  return agABTHashContainer(c, hash);
}

template <class Object>
inline int agABTHash(const AgArray<Object> &c) {
  return agABTHashContainer(c, 0);
}


#endif
