#ifndef AG_STRING_BASE_H
#define AG_STRING_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <assert.h>
#include <string.h>
#include "agref.h"

class AgStringBase : public AgReferenceCounter {
  int bufferSize;
  char *buffer;
  int count;
public:
  AgStringBase();
  AgStringBase(int);
  AgStringBase(const char *);
  AgStringBase(const AgStringBase &);
  AgStringBase &operator = (const AgStringBase &b);
  ~AgStringBase() {delete [] buffer;}
  operator const char *() const {if (count) return buffer; else return NULL;}
  int size() const {return count;}
  char &operator [](int x) const {
    assert((unsigned) x < (unsigned) count); return buffer[x];}
  AgStringBase &append(int c);
  AgStringBase &append(const char *);
  AgStringBase &append(const char *, int);
  AgStringBase &reset() {buffer[count = 0] = 0; return *this;}

  int operator <  (const char *s) const {return strcmp(buffer, s) < 0;}
  int operator <= (const char *s) const {return strcmp(buffer, s) <= 0;}
  int operator >  (const char *s) const {return strcmp(buffer, s) > 0;}
  int operator >= (const char *s) const {return strcmp(buffer, s) >= 0;}
  int operator == (const char *s) const {return strcmp(buffer, s) == 0;}
  int operator != (const char *s) const {return strcmp(buffer, s) != 0;}
};

int agABTHash(const char *, int = 0);

inline int agABTHash(const AgStringBase &string, int hashcode) {
  return agABTHash((const char *) string, hashcode);
}

#endif
