#ifndef DCI_H_1027348674
#define DCI_H_1027348674

#ifndef AG_OBJECT_WRAPPER_DEFINED
#define AG_OBJECT_WRAPPER_DEFINED

#ifndef AG_PLACEMENT_DELETE_REQUIRED
#if _MSC_VER >= 1200 || __INTEL_COMPILER
#define AG_PLACEMENT_DELETE_REQUIRED 1
#endif
#endif

template <class Object>
class AgObjectWrapper {
  Object object;
public:
  void operator delete(void *) {}
  void *operator new(size_t, void *p) { return p;}
#if AG_PLACEMENT_DELETE_REQUIRED
  void operator delete(void *, void *) { }
#endif
  AgObjectWrapper(const Object &o) : object(o) {}
  ~AgObjectWrapper() {}
  operator Object &() {return object;}
};

#endif

typedef union {
  double alignment;
  char ag_vt_2[sizeof(int)];
  char ag_vt_3[sizeof(long)];
  char ag_vt_4[sizeof(AgObjectWrapper<AgString >)];
  char ag_vt_5[sizeof(AgObjectWrapper<Value >)];
  char ag_vt_6[sizeof(AgObjectWrapper<CodeFragment >)];
  char ag_vt_7[sizeof(AgObjectWrapper<AgStack<CodeFragment> >)];
  char ag_vt_8[sizeof(AgObjectWrapper<Constant >)];
  char ag_vt_9[sizeof(Opcode)];
  char ag_vt_10[sizeof(double)];
} dci_vs_type;

typedef enum {
  dci_white_space_token = 1, dci_script_token = 7, dci_statement_list_token,
  dci_eof_token, dci_statement_token, dci_open_statement_token,
  dci_closed_statement_token, dci_if_condition_token, dci_WHILE_token = 15,
  dci_expression_token, dci_FOR_token = 18, dci_lvalue_token,
  dci_for_increment_token = 21, dci_simple_statement_token = 23,
  dci_REPEAT_token = 25, dci_compound_statement_token = 28,
  dci_dump_statement_token = 30, dci_print_statement_token,
  dci_simple_expression_token = 45, dci_relational_op_token, dci_term_token,
  dci_additive_op_token, dci_unary_expression_token,
  dci_multiplicative_op_token, dci_factor_token, dci_unary_op_token = 53,
  dci_primary_token, dci_constant_token = 58, dci_function_call_token,
  dci_optional_arg_list_token = 62, dci_arg_list_token, dci_string_token,
  dci_space_token = 82, dci_letter_token = 93, dci_simple_real_token = 95,
  dci_signed_exponent_token = 97, dci_integer_part_token,
  dci_fraction_part_token = 100, dci_decimal_integer_token,
  dci_hybrid_integer_token, dci_octal_integer_token,
  dci_exponent_token = 105, dci_digit_token, dci_hex_integer_token,
  dci_hex_digit_token = 114, dci_s_char_sequence_token = 117,
  dci_s_char_token, dci_escape_sequence_token = 120,
  dci_simple_escape_sequence_token, dci_octal_escape_sequence_token,
  dci_hexadecimal_escape_sequence_token, dci_one_octal_token = 135,
  dci_two_octal_token, dci_three_octal_token,
  dci_hexadecimal_digit_token = 140, dci_c_char_token = 142,
  dci_name_token = 161, dci_integer_token = 179, dci_real_token,
  dci_character_constant_token, dci_string_element_token = 199
} dci_token_type;

typedef struct dci_pcb_struct{
  dci_token_type token_number, reduction_token, error_frame_token;
  int input_code;
  int input_value;
  int line, column;
  int ssx, sn, error_frame_ssx;
  int drt, dssx, dsn;
  int ss[128];
  dci_vs_type vs[128];
  int ag_ap;
  char *error_message;
  char read_flag;
  char exit_flag;
  FileLocation input_context;
  FileLocation cs[128];
  int bts[128], btsx;
  const unsigned char * pointer;
  const unsigned char * la_ptr;
  const unsigned char *key_sp;
  int save_index, key_state;
  char ag_msg[82];
/*  Line 100, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
    // Maps symbol names to variables
    AgDictionary<AgString> &dictionary;                        // symbol table

    // table of constants
    AgDictionary<Constant> constants;

    // Constructor for pcb
    dci_pcb_struct(AgDictionary<AgString> &d, const char *text);

    // Functions used during parsing
    void reportError();
    void reportError(const char *msg);
    int idName(const AgString &);
    CodeFragment code(Opcode, const AgString &);
    CodeFragment code(Opcode, const Constant &);
    CodeFragment codeCall(const AgString &, AgStack<CodeFragment> &);
  } dci_pcb_type;

#ifndef PRULE_CONTEXT
#define PRULE_CONTEXT(pcb)  (&((pcb).cs[(pcb).ssx]))
#define PERROR_CONTEXT(pcb) ((pcb).cs[(pcb).error_frame_ssx])
#define PCONTEXT(pcb)       ((pcb).cs[(pcb).ssx])
#endif

#ifndef AG_RUNNING_CODE_CODE
/* PCB.exit_flag values */
#define AG_RUNNING_CODE         0
#define AG_SUCCESS_CODE         1
#define AG_SYNTAX_ERROR_CODE    2
#define AG_REDUCTION_ERROR_CODE 3
#define AG_STACK_ERROR_CODE     4
#define AG_SEMANTIC_ERROR_CODE  5
#endif
void init_dci(dci_pcb_type *);
void dci(dci_pcb_type *);

CodeFragment dci_value(dci_pcb_type *);
#endif

